EPlayer = {}

function EPlayer.new()
	return ({
		renderPosition = CPosition.new(10, 0),
		velocity = CVelocity.new(0, 0),
		tileMapCollisionRectangle = CRectangle.new(0, 0, 6, 6),
		renderRectangle = CRectangle.new(0, 0, TILE_SIZE, TILE_SIZE),
		jump = CJump.new()
	})
end

function EPlayer.update(e, dt)
	e.velocity.x = 0
	
	if love.keyboard.isDown("d") then
		e.velocity.x = 5 * TILE_SIZE
	elseif love.keyboard.isDown("a") then
		e.velocity.x = -5 * TILE_SIZE
	end
	
	if love.keyboard.isDown(" ") and e.jump.onGround then
		e.velocity.y = -e.jump.speed
		e.jump.onGround = false
	end
	
	EPlayer.updateY(e, dt)
	EPlayer.updateX(e, dt)
	
	e.renderRectangle.x = e.renderPosition.x
	e.renderRectangle.y = e.renderPosition.y
end

function EPlayer.draw(e)
love.graphics.setColor(255, 255, 255)
	love.graphics.rectangle(
		"fill",
		e.renderRectangle.x,
		e.renderRectangle.y,
		e.renderRectangle.w,
		e.renderRectangle.h
	)
	love.graphics.setColor(255, 0, 0)
	love.graphics.rectangle(
		"fill",
		e.tileMapCollisionRectangle.x,
		e.tileMapCollisionRectangle.y,
		e.tileMapCollisionRectangle.w,
		e.tileMapCollisionRectangle.h
	)
	love.graphics.setColor(255, 255, 255)
end

function EPlayer.updateX(e, dt)
	local dx = e.velocity.x * dt
	
	e.tileMapCollisionRectangle.y = e.renderPosition.y + 1
	
	-- moving right
	if dx > 0 then
		e.tileMapCollisionRectangle.x = e.renderPosition.x + 2 + dx
	-- moving left
	elseif dx < 0 then
		e.tileMapCollisionRectangle.x = e.renderPosition.x + dx
	-- not moving
	else
		return
	end
	
	local collidingTiles = World:getCollidingTiles(e.tileMapCollisionRectangle)
	local coll = false
	local collideX = -1
	local currentDefinition
	
	for i, currentInstance in pairs(collidingTiles) do
		currentDefinition = TileDictionary:getDefinitionOf(currentInstance.id)
		
		if currentDefinition.collide then
			collideX = currentInstance.x
			coll = true
			break
		end
	end
	
	if coll then
		e.velocity.x = 0
		-- moving right
		if dx > 0 then
			e.renderPosition.x = collideX - e.renderRectangle.w
		-- moving left
		else
			e.renderPosition.x = collideX + TILE_SIZE
		end
	else
		e.renderPosition.x = e.renderPosition.x + dx
	end
end

function EPlayer.updateY(e, dt)
	if not e.jump.onGround then
		e.velocity.y = math.min(
			e.velocity.y + (World:getGravity() * dt),
			World:getTerminalVelocity()
		)
	end
	
	local dy = e.velocity.y * dt
	
	e.tileMapCollisionRectangle.x = e.renderPosition.x + 1
	
	-- moving down
	if dy > 0 then
		e.tileMapCollisionRectangle.y = e.renderPosition.y + 2 + dy
	-- moving up
	elseif dy < 0 then
		e.tileMapCollisionRectangle.y = e.renderPosition.y + dy
	end
	
	local collidingTiles = World:getCollidingTiles(e.tileMapCollisionRectangle)
	local coll = false
	local collideY = -1
	local currentDefinition
	
	for i, currentInstance in pairs(collidingTiles) do
		currentDefinition = TileDictionary:getDefinitionOf(currentInstance.id)
		
		if currentDefinition.collide then
			collideY = currentInstance.y
			coll = true
			break
		end
	end
	
	if coll then
		e.velocity.y = 0
		-- moving down
		if dy > 0 then
			e.renderPosition.y = collideY - e.renderRectangle.h
			e.jump.onGround = true
		-- moving up
		else
			e.renderPosition.y = collideY + TILE_SIZE
		end
	else
		if e.jump.onGround then
			e.jump.onGround = false
		end
		e.renderPosition.y = e.renderPosition.y + dy
	end
end