World = {}

function World:init()
	TileDictionary:init()
	TileDrawer:init()

	self.tileData = {}
	
	self.width = 32
	self.height = 24
	
	self.gravity = 9.81 * TILE_SIZE
	self.terminalVelocity = 6 * TILE_SIZE
	
	for y = 0, self.height - 1 do
		self.tileData[y] = {}
		for x = 0, self.width - 1 do
			local id = TILE_AIR
			
			if y == 15 then
				id = TILE_GRASS
			elseif y > 15 then
				id = TILE_DIRT
			end
		
			self.tileData[y][x] = TileInstance.new(
				id,
				x * TILE_SIZE,
				y * TILE_SIZE
			)
		end
	end
end

function World:update(dt)

end

function World:draw()
	for y, row in pairs(self.tileData) do
		for x, tile in pairs(self.tileData[y]) do
			TileDrawer:drawTileInstance(tile)
		end
	end
end

function World:getCollidingTiles(rectangle)
	local startX = math.floor(rectangle.x / TILE_SIZE)
	local startY = math.floor(rectangle.y / TILE_SIZE)
	local endX = math.floor((rectangle.x + rectangle.w) / TILE_SIZE)
	local endY = math.floor((rectangle.y + rectangle.h) / TILE_SIZE)
	
	assert(
		startX > -1 and startY > -1 and endX < self.width and endY < self.height,
		"World:getCollidingTiles() - requested rectangle outside of world bounds"
	)
	
	local tmp = {}
	
	for y = startY, endY do
		for x = startX, endX do
			table.insert(tmp, self.tileData[y][x])
		end
	end
	
	return tmp
end

function World:tileAt(x, y)
	local gx = math.floor(x / TILE_SIZE)
	local gy = math.floor(y / TILE_SIZE)
	
	assert(
		gx > -1 and gy > -1 and gx < self.width and gy < self.height,
		"World:tileAt() - requested point outside of world bounds"
	)
	
	return self.tileData[gy][gx]
end

function World:getGravity()
	return self.gravity
end

function World:getTerminalVelocity()
	return self.terminalVelocity
end